
# q-expansion of newform 240.3.c.d, downloaded from the LMFDB on 09 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 240
weight = 3
poly_data = [1, 0, 3, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 4, 0, 0], 1], [[3, 4, 2, 2], 1], [[3, -4, 2, -2], 1]]

hecke_ring_character_values = [[31, [1, 0, 0, 0]], [181, [1, 0, 0, 0]], [161, [-1, 0, 0, 0]], [97, [-1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, 0, 0, 1], [0, 1, 0, -1], [0, 0, -2, 2], [0, -2, -1, 1], [0, 0, -3, 3], [0, 0, 7, 7], [6, 0, 0, 0], [0, 0, 1, 1], [0, 6, 3, -3], [-34, 0, 0, 0], [0, 0, 11, -11], [0, -4, -2, 2], [0, 0, -7, 7], [0, 0, 1, 1], [0, 0, -9, -9], [0, 22, 11, -11], [74, 0, 0, 0], [0, 0, 23, -23], [0, 12, 6, -6], [0, 0, -14, 14], [78, 0, 0, 0], [0, 0, -23, -23], [0, 4, 2, -2], [0, 0, -8, 8], [0, 34, 17, -17], [0, 0, -26, 26], [0, 0, 33, 33], [74, 0, 0, 0], [0, 0, -9, -9], [0, 0, 4, -4], [0, 18, 9, -9], [0, 0, 39, 39], [118, 0, 0, 0], [0, -22, -11, 11], [-34, 0, 0, 0], [0, 0, -23, 23], [0, 0, -17, 17], [0, 0, -15, -15], [0, 0, -17, -17], [0, 58, 29, -29], [-166, 0, 0, 0], [0, -48, -24, 24], [0, 0, -8, 8], [0, 0, -1, -1], [-114, 0, 0, 0], [6, 0, 0, 0], [0, 0, 68, -68], [0, 0, -55, -55], [154, 0, 0, 0], [0, 0, -41, -41], [0, -40, -20, 20], [-206, 0, 0, 0], [0, 6, 3, -3], [0, 0, -9, -9], [0, 0, -47, -47], [0, 30, 15, -15], [398, 0, 0, 0], [0, 0, -73, 73], [0, -12, -6, 6], [0, 0, 13, -13], [0, 0, 87, 87], [0, 0, 123, -123], [0, -108, -54, 54], [0, 0, 142, -142], [0, 0, -137, -137], [-202, 0, 0, 0], [0, 0, 92, -92], [0, 0, 57, 57], [-118, 0, 0, 0], [0, 0, 7, 7], [0, 12, 6, -6], [0, 0, 88, -88], [0, 0, -33, 33], [-394, 0, 0, 0], [0, 0, 17, 17], [0, -62, -31, 31], [0, 0, -163, 163], [0, 40, 20, -20], [-206, 0, 0, 0], [0, -46, -23, 23], [-38, 0, 0, 0], [0, 136, 68, -68], [0, 0, -68, 68], [366, 0, 0, 0], [0, 0, 129, 129], [0, 96, 48, -48], [0, 0, 26, -26], [0, -114, -57, 57], [0, 0, 24, -24], [0, 0, 33, 33], [0, 128, 64, -64], [0, 0, -162, 162], [0, 30, 15, -15], [486, 0, 0, 0], [0, 0, 177, 177], [0, 6, 3, -3], [0, 220, 110, -110], [0, 0, 73, -73], [-86, 0, 0, 0], [0, 0, -41, 41], [0, 0, -81, -81], [0, 0, -223, -223], [0, -140, -70, 70], [-394, 0, 0, 0], [0, 0, 152, -152], [0, 0, -159, -159], [0, 0, 135, 135], [0, -12, -6, 6], [434, 0, 0, 0], [0, 0, 164, -164], [0, 0, 211, -211], [0, 0, 103, 103], [1046, 0, 0, 0], [46, 0, 0, 0], [0, -256, -128, 128], [0, 0, -201, 201], [0, 0, 129, 129], [0, 0, -241, -241], [0, -182, -91, 91], [1082, 0, 0, 0], [0, 0, -264, 264], [0, 0, -137, -137], [0, 0, -71, -71], [-922, 0, 0, 0], [0, -106, -53, 53], [-966, 0, 0, 0], [0, 248, 124, -124], [0, 0, -102, 102], [0, 0, 41, -41], [-1082, 0, 0, 0], [0, 0, -255, -255], [958, 0, 0, 0], [0, 0, -193, 193], [0, -252, -126, 126], [-1326, 0, 0, 0], [0, 0, -33, -33], [0, 0, 283, -283], [0, 0, -137, -137], [0, -276, -138, 138], [-874, 0, 0, 0], [0, 58, 29, -29], [0, 0, -342, 342], [0, 0, -103, -103], [1002, 0, 0, 0], [0, -68, -34, 34], [0, 0, -193, 193], [0, 0, -201, -201], [278, 0, 0, 0], [0, 0, 129, 129], [0, 0, 157, -157], [0, 120, 60, -60], [0, 0, -237, 237], [0, 0, -63, -63], [0, 0, 129, -129], [0, -72, -36, 36], [-1314, 0, 0, 0], [0, 336, 168, -168], [0, 0, 322, -322], [0, -82, -41, 41], [0, 0, -375, -375], [0, 0, -185, -185], [0, 0, 134, -134], [0, 30, 15, -15], [0, 0, 39, 39], [0, 0, -415, -415], [1646, 0, 0, 0], [0, 0, 291, -291]]
